from . import *

from typing import *
import time

from base_aux.funcs import *
from base_aux.classes import *


# =====================================================================================================================
# FIXME: this is just an attempt to replace simple dict!!!
class CmdSchema:
    NAME: str
    SCHEMA: Any | Valid | ValueNotExist = ValueNotExist
    TIMEOUT: float = 0
    DEFAULT: Any | ValueNotExist = ValueNotExist

    __value: Any = ValueNotExist

    # todo: add init

    @property
    def value(self) -> Any:
        pass
        # if self.__value == ValueNotExist:
        #     result = self.

        # return result

    @value.setter
    def value(self, other) -> None:
        self.__value = other

    def __init__(self, name):
        pass

    def __str__(self) -> str:
        return self.output()

    def output(self, value: Any | ValueNotExist = ValueNotExist) -> str:
        if value == ValueNotExist:
            value = self.DEFAULT

        if self.SCHEMA == ValueNotExist:
            return str(value)
        else:
            result = ValidAux.get_result_or_exx(self.SCHEMA, args=value)
            return str(result)


class CmdsSchema:
    """
    CREATED SPECIALLY FOR
    ---------------------
    serialClient - to keep default timeouts
    serialServer - to replace simple dictSchema my full needs about schemas
    """
    # default cmds
    # CMD1: CmdSchema = CmdSchema("CMD1", )

    # todo: add getattr by anyRegister
    # todo: add iterate

    def __init__(self, *schemas: tuple | CmdSchema) -> None:
        self.update(*schemas)

    def update(self, *schemas: tuple | CmdSchema) -> None:
        """
        overwrite existed schemas!
        """
        for schema in schemas:
            if not isinstance(schema, CmdSchema):
                schema = CmdSchema(*schema)

            setattr(self, schema.NAME, schema)


# =====================================================================================================================
# =====================================================================================================================
# =====================================================================================================================
# =====================================================================================================================
# =====================================================================================================================
# =====================================================================================================================
class SerialServer_Example(SerialServer_Base):
    PARAMS = {
        "VAR": "",

        "STR": "str",
        "QUOTE": "str'",

        "BLANC": "",
        "ZERO": 0,

        "NONE": None,
        "TRUE": True,
        "FALSE": False,

        "INT": 1,
        "FLOAT": 1.1,

        "CALL": time.time,
        "EXX": time.strftime,

        "LIST": [0, 1, 2],
        "LIST_2": [[11]],
        "_SET": {0, 1, 2},
        "DICT_SHORT": {1: 11},
        "DICT_SHORT_2": {"HEllo": {1: 11}},
        "DICT": {
            1: 111,
            "2": 222,
            3: {
                1: 31,
                2: 32,
            },
        },
        "UNIT": ValueUnit(1, unit="V"),
        "VARIANT": ValueVariants(220, variants=[220, 380]),
    }

    def cmd__upper(self, line_parsed: CmdArgsKwargs_ByStr) -> TYPE__CMD_RESULT:
        # usefull for tests
        return line_parsed.ORIGINAL.upper()

    def cmd__lower(self, line_parsed: CmdArgsKwargs_ByStr) -> TYPE__CMD_RESULT:
        return line_parsed.ORIGINAL.lower()

    def cmd__cmd(self, line_parsed: CmdArgsKwargs_ByStr) -> TYPE__CMD_RESULT:
        # NOTE: NONE is equivalent for SUCCESS
        # do smth
        pass

    def cmd__cmd_no_line(self) -> TYPE__CMD_RESULT:
        # NOTE: NONE is equivalent for SUCCESS
        # do smth
        pass

    # -----------------------------------------------------------------------------------------------------------------
    def script__script1(self, line_parsed: CmdArgsKwargs_ByStr) -> TYPE__CMD_RESULT:
        # do smth
        pass


# =====================================================================================================================
if __name__ == "__main__":
    SerialServer_Example().run()


# =====================================================================================================================



from typing import *
import json
import re

from base_aux.funcs import TYPE__ELEMENTARY, args__ensure_tuple
from base_aux.objects import TypeChecker


# =====================================================================================================================
class Text:
    SOURCE: str = None

    def __init__(self, source: Optional[str] = None):
        if source is not None:
            self.SOURCE = source

    # -----------------------------------------------------------------------------------------------------------------
    def prepare__json_loads(self, source: Optional[str] = None) -> str:
        """
        GOAL
        ----
        replace pytonic values (usually created by str(Any)) before attempting to apply json.loads to get original python objects
        so it just same process as re.sub by one func for several values

        SPECIALLY CREATED FOR
        ---------------------
        try_convert_to_object
        """
        insource = source is None
        if insource:
            source = self.SOURCE
        result = source
        if isinstance(source, str):
            result = self.sub__words(
                rules = [
                    (r"True", "true"),
                    (r"False", "false"),
                    (r"None", "null"),
                ]
            )
        if insource:
            self.SOURCE = result
        return result

    def prepare__requirements(self, source: Optional[str] = None) -> str:
        """
        GOAL
        ----
        replace pytonic values (usually created by str(Any)) before attempting to apply json.loads to get original python objects
        so it just same process as re.sub by one func for several values

        SPECIALLY CREATED FOR
        ---------------------
        try_convert_to_object
        """
        insource = source is None
        if insource:
            source = self.SOURCE

        result = source
        result = self.clear__cmts(result)
        result = self.clear__blank_lines(result)
        if insource:
            self.SOURCE = result
        return result

    # -----------------------------------------------------------------------------------------------------------------
    def sub__word(self, word_pat: str, new: str = "", source: Optional[str] = None) -> str:
        """
        GOAL
        ----
        replace exact word(defined by pattern) in text.
        WORD means syntax word!

        SPECIALLY CREATED FOR
        ---------------------
        prepare_for_json_parsing
        """
        insource = source is None
        if insource:
            source = self.SOURCE

        word_pat = r"\b" + word_pat + r"\b"
        result = re.sub(word_pat, new, source)

        if insource:
            self.SOURCE = result
        return result

    def sub__words(self, rules: list[tuple[str, str]], source: Optional[str] = None) -> str:
        insource = source is None
        if insource:
            source = self.SOURCE

        result = source
        for work_pat, new in rules:
            result = self.sub__word(work_pat, new, result)
        return result

    # =================================================================================================================
    def clear__blank_lines(
            self,
            source: Optional[str] = None,
    ) -> str:
        insource = source is None
        if insource:
            source = self.SOURCE

        result = source
        if isinstance(source, str):
            result = re.sub(pattern=r"^\s*$", repl="", string=result, flags=re.MULTILINE)

        if insource:
            self.SOURCE = result
        return result

    def clear__cmts(
            self,
            source: Optional[str] = None,
    ) -> str:
        insource = source is None
        if insource:
            source = self.SOURCE

        result = source
        if isinstance(source, str):
            result = re.sub(pattern=r"\s*\#.*$", repl="", string=result, flags=re.MULTILINE)

        if insource:
            self.SOURCE = result
        return result

    # =================================================================================================================
    def lines__split(
            self,
            source: Optional[str] = None,
    ) -> list[str]:
        insource = source is None
        if insource:
            source = self.SOURCE

        result = source.splitlines()
        return result

    def lines__strip(
            self,
            lines: list[str] = None,
    ) -> list[str]:
        insource = lines is None
        if insource:
            lines = self.lines__split()

        result = []
        for line in lines:
            result.append(line.strip())
        return result

    def lines__clear_blank(
            self,
            lines: list[str] = None,
    ) -> list[str]:
        insource = lines is None
        if insource:
            lines = self.lines__split()

        result = []
        for line in lines:
            if line:
                result.append(line)
        return result

    # =================================================================================================================
    def try_convert_to_object(self, source: Optional[str] = None) -> TYPE__ELEMENTARY | str:
        """
        GOAL
        ----
        create an elementary object from text.
        or return source

        NOTE
        ----
        by now it works correct only with single elementary values like INT/FLOAT/BOOL/NONE
        for collections it may work but may not work correctly!!! so use it by your own risk and conscious choice!!
        """
        # FIXME: this is not work FULL and CORRECT!!!! need FIX!!!
        insource = source is None
        if insource:
            source = self.SOURCE

        # PREPARE SOURCE ----------
        source_original = source
        source = self.prepare__json_loads(source)

        # WORK --------------------
        try:
            source_elementary = json.loads(source)
            return source_elementary
        except Exception as exx:
            print(f"{exx!r}")
            return source_original

    # -----------------------------------------------------------------------------------------------------------------
    def find_by_pats(
            self,
            patterns: list[str] | str,
            source: Optional[str] = None,
    ) -> list[str]:
        """
        GOAL
        ----
        find all pattern values in text

        NOTE
        ----
        if pattern have group - return group value (as usual)
        """
        insource = source is None
        if insource:
            source = self.SOURCE

        result = []
        patterns = args__ensure_tuple(patterns)

        for pat in patterns:
            result_i = re.findall(pat, source)
            for value in result_i:
                value: str
                if value == "":
                    continue
                value = value.strip()
                if value not in result:
                    result.append(value)
        return result

    # -----------------------------------------------------------------------------------------------------------------
    def requirements__get_list(
            self,
            source: Optional[str] = None,
    ) -> list[str]:
        """
        GOAL
        ----
        get list of required modules (actually full lines stripped and commentsCleared)

        SPECIALLY CREATED FOR
        ---------------------
        setup.py install_requires
        """
        insource = source is None
        if insource:
            source = self.SOURCE

        result = source
        result = self.prepare__requirements(result)

        result = self.lines__split(result)
        result = self.lines__strip(result)
        result = self.lines__clear_blank(result)

        return result


# =====================================================================================================================



from base_aux.testplans import *
from base_aux.funcs import *
from base_aux.classes import *


# =====================================================================================================================
class TestCase(TestCaseBase):
    ASYNC = True
    DESCRIPTION = "ptb"

    @classmethod
    def startup__cls__wrapped(cls) -> TYPE__RESULT_W_NORETURN:
        # return True
        result_chain = ValidChains(
            [
                Valid(value_link=hasattr(cls, "DEVICES__BREEDER_CLS"), name="hasattr DEVICES__CLS"),
                Valid(value_link=hasattr(cls.DEVICES__BREEDER_CLS, "ATC"), name="hasattr ATC"),
                Valid(value_link=cls.DEVICES__BREEDER_CLS.ATC.connect, name="ATC.connect()"),
            ],
        )
        return result_chain

    def startup__wrapped(self) -> TYPE__RESULT_W_NORETURN:
        result = ValidChains(
            [
                Valid(value_link=self.DEVICES__BREEDER_INST.DUT.connect__only_if_address_resolved, name="DUT.connect__only_if_address_resolved"),
            ],
        )
        return result

    def run__wrapped(self) -> TYPE__RESULT_W_NORETURN:
        # time.sleep(0.1)
        result = ValidChains(
            [
                Valid(value_link=self.DEVICES__BREEDER_INST.DUT.connect__only_if_address_resolved, name="DUT.connect__only_if_address_resolved"),
            ],
        )
        return result


# =====================================================================================================================
